// From the software distribution accompanying the textbook
// "A Practical Introduction to Data Structures and Algorithm Analysis,
// Third Edition (C++)" by Clifford A. Shaffer.
// Source code Copyright (C) 2007-2011 by Clifford A. Shaffer.

// This is the file to include in your code if you want access to the
// complete AQueue template class

#include "queue.h"

template <typename E>
class AQueue: public Queue<E>
{
    private:
        int maxSize;            // Maximum size of queue
        int drifting_front;     // Index of drifting_front element
        int drifting_rear;      // Index of drifting_rear element
        E *listArray;           // Array holding queue elements

    public:
        AQueue(int queue_size = defaultSize)
        {
            // Make list array one position larger for empty slot
            maxSize = queue_size + 1;
            drifting_rear = 0;
            drifting_front = 1;
            listArray = new E[maxSize];
        }

        ~AQueue()
        {
            delete[] listArray;
        }

        void clear()
        {
            // Deleting in array structures is just changing an index.
            drifting_rear = 0;
            drifting_front = 1;
        }

        void enqueue(const E &it)
        {
            Assert(((drifting_rear + 2) % maxSize) != drifting_front, "Queue is full");

            drifting_rear = (drifting_rear + 1) % maxSize;       // Circular increment
            listArray[drifting_rear] = it;
        }

        E dequeue()
        {
            Assert(length() != 0, "Queue is empty");

            E it = listArray[drifting_front];
            drifting_front = (drifting_front + 1) % maxSize;    // Circular increment

            return it;
        }

        const E & frontValue() const
        {
            Assert(length() != 0, "Queue is empty");

            return listArray[drifting_front];
        }

        int length() const
        {
            return ((drifting_rear + maxSize) - drifting_front + 1) % maxSize;
        }
};

