// From the software distribution accompanying the textbook
// "A Practical Introduction to Data Structures and Algorithm Analysis,
// Third Edition (C++)" by Clifford A. Shaffer.
// Source code Copyright (C) 2007-2011 by Clifford A. Shaffer.

// This is the file to include in your code if you want access to the
// complete LQueue template class

// Include the Node class
#include "node.h"
#include "queue.h"

// Implementations for Nodeed queue function members
// Linked queue implementation
template <typename E>
class LQueue: public Queue<E>
{
    private:
        Node<E> *pFrontN;       // Pointer to front queue node
        Node<E> *pRearN;        // Pointer to rear queue node
        int queue_size;                // Number of elements in queue

    public:
        // empty head node
        LQueue(int sz = defaultSize)
        {
            pFrontN = pRearN = new Node<E>;
            queue_size = 0;
        }

        ~LQueue()
        {
            clear();
            delete pFrontN;
        }

        void clear()
        {
            Node<E> *temp;

            while(pFrontN->next != nullptr)
            {
                temp = pFrontN->next;
                pFrontN->next = pFrontN->next->next;
                delete temp;
            }

            pRearN = pFrontN;
            queue_size = 0;
        }

        // Put element on rear
        void enqueue(const E &it)
        {
            pRearN->next = new Node<E>(it, nullptr);
            pRearN = pRearN->next;
            queue_size++;
        }

        // Remove element from front
        E dequeue()
        {
            Assert(queue_size != 0, "Queue is empty");

            E it = pFrontN->next->element;  // Store dequeued value
            Node<E> *ltemp = pFrontN->next; // Hold dequeued Node
            pFrontN->next = pFrontN->next->next;       // Advance front

            if(pRearN == ltemp)
                pRearN = pFrontN; // Dequeue last element

            delete ltemp;                    // Delete Node
            queue_size--;

            return it;                       // Return element value
        }

        // Get front element
        const E & frontValue() const
        {
            Assert(queue_size != 0, "Queue is empty");

            return pFrontN->next->element;
        }

        virtual int length() const
        {
            return queue_size;
        }
};

