// From the software distribution accompanying the textbook
// "A Practical Introduction to Data Structures and Algorithm Analysis,
// Third Edition (C++)" by Clifford A. Shaffer.
// Source code Copyright (C) 2007-2011 by Clifford A. Shaffer.

// This is the file to include in your code if you want access to the
// complete LStack template class

// Include the Node class
#include "node.h"

// First, get the declaration for the base stack class
#include "stack.h"

// This is the declaration for LStack.
// Nodeed stack implementation
template <typename E>
class LStack: public Stack<E>
{
    private:
        // Pointer to first element
        Node<E> *pTop;

        // Number of elements
        int size;

    public:
        // Constructor
        LStack(int sz = defaultSize)
        {
            pTop = nullptr;
            size = 0;
        }

        // Destructor
        ~LStack()
        {
            clear();
        }

        // Reinitialize
        void clear()
        {
            // Delete Node nodes
            Node<E> *temp = pTop;
            while (pTop != nullptr)
            {
                temp = pTop;
                pTop = pTop->next;
                delete temp;
            }
            size = 0;
        }

        // Put "it" on stack
        void push(const E& it)
        {
            pTop = new Node<E>(it, pTop);
            size++;
        }

        // Remove "it" from stack
        E pop()
        {
            Assert(pTop != nullptr, "Stack is empty");
            E it = pTop->element;
            Node<E> *ltemp = pTop->next;
            delete pTop;
            pTop = ltemp;
            size--;
            return it;
        }

        // Return top value
        const E& peek() const
        {
            Assert(pTop != 0, "Stack is empty");
            return pTop->element;
        }

        // Return length
        int length() const
        {
            return size;
        }
};
